# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.http_response import AsyncHttpResponse, HttpResponse
from ...core.jsonable_encoder import jsonable_encoder
from ...core.pagination import AsyncPager, SyncPager
from ...core.pydantic_utilities import parse_obj_as
from ...core.request_options import RequestOptions
from ...core.serialization import convert_and_respect_annotation_metadata
from ..errors.bad_request_error import BadRequestError
from ..types.error_response import ErrorResponse
from ..types.posted_builtin_tool import PostedBuiltinTool
from ..types.posted_config_prompt_spec import PostedConfigPromptSpec
from ..types.posted_ellm_model import PostedEllmModel
from ..types.posted_event_message_specs import PostedEventMessageSpecs
from ..types.posted_language_model import PostedLanguageModel
from ..types.posted_nudge_spec import PostedNudgeSpec
from ..types.posted_timeout_specs import PostedTimeoutSpecs
from ..types.posted_user_defined_tool_spec import PostedUserDefinedToolSpec
from ..types.posted_webhook_spec import PostedWebhookSpec
from ..types.return_config import ReturnConfig
from ..types.return_paged_configs import ReturnPagedConfigs
from ..types.voice_ref import VoiceRef

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawConfigsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list_configs(
        self,
        *,
        page_number: typing.Optional[int] = None,
        page_size: typing.Optional[int] = None,
        restrict_to_most_recent: typing.Optional[bool] = None,
        name: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[ReturnConfig, ReturnPagedConfigs]:
        """
        Fetches a paginated list of **Configs**.

        For more details on configuration options and how to configure EVI, see our [configuration guide](/docs/speech-to-speech-evi/configuration).

        Parameters
        ----------
        page_number : typing.Optional[int]
            Specifies the page number to retrieve, enabling pagination.

            This parameter uses zero-based indexing. For example, setting `page_number` to 0 retrieves the first page of results (items 0-9 if `page_size` is 10), setting `page_number` to 1 retrieves the second page (items 10-19), and so on. Defaults to 0, which retrieves the first page.

        page_size : typing.Optional[int]
            Specifies the maximum number of results to include per page, enabling pagination. The value must be between 1 and 100, inclusive.

            For example, if `page_size` is set to 10, each page will include up to 10 items. Defaults to 10.

        restrict_to_most_recent : typing.Optional[bool]
            By default, `restrict_to_most_recent` is set to true, returning only the latest version of each config. To include all versions of each config in the list, set `restrict_to_most_recent` to false.

        name : typing.Optional[str]
            Filter to only include configs with this name.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[ReturnConfig, ReturnPagedConfigs]
            Success
        """
        page_number = page_number if page_number is not None else 0

        _response = self._client_wrapper.httpx_client.request(
            "v0/evi/configs",
            base_url=self._client_wrapper.get_environment().base,
            method="GET",
            params={
                "page_number": page_number,
                "page_size": page_size,
                "restrict_to_most_recent": restrict_to_most_recent,
                "name": name,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ReturnPagedConfigs,
                    parse_obj_as(
                        type_=ReturnPagedConfigs,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.configs_page
                _has_next = True
                _get_next = lambda: self.list_configs(
                    page_number=page_number + 1,
                    page_size=page_size,
                    restrict_to_most_recent=restrict_to_most_recent,
                    name=name,
                    request_options=request_options,
                )
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create_config(
        self,
        *,
        evi_version: str,
        name: str,
        builtin_tools: typing.Optional[typing.Sequence[typing.Optional[PostedBuiltinTool]]] = OMIT,
        ellm_model: typing.Optional[PostedEllmModel] = OMIT,
        event_messages: typing.Optional[PostedEventMessageSpecs] = OMIT,
        language_model: typing.Optional[PostedLanguageModel] = OMIT,
        nudges: typing.Optional[PostedNudgeSpec] = OMIT,
        prompt: typing.Optional[PostedConfigPromptSpec] = OMIT,
        timeouts: typing.Optional[PostedTimeoutSpecs] = OMIT,
        tools: typing.Optional[typing.Sequence[typing.Optional[PostedUserDefinedToolSpec]]] = OMIT,
        version_description: typing.Optional[str] = OMIT,
        voice: typing.Optional[VoiceRef] = OMIT,
        webhooks: typing.Optional[typing.Sequence[typing.Optional[PostedWebhookSpec]]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ReturnConfig]:
        """
        Creates a **Config** which can be applied to EVI.

        For more details on configuration options and how to configure EVI, see our [configuration guide](/docs/speech-to-speech-evi/configuration).

        Parameters
        ----------
        evi_version : str
            EVI version to use. Only versions `3` and `4-mini` are supported.

        name : str
            Name applied to all versions of a particular Config.

        builtin_tools : typing.Optional[typing.Sequence[typing.Optional[PostedBuiltinTool]]]
            Built-in tool specification for a Config.

        ellm_model : typing.Optional[PostedEllmModel]

        event_messages : typing.Optional[PostedEventMessageSpecs]

        language_model : typing.Optional[PostedLanguageModel]

        nudges : typing.Optional[PostedNudgeSpec]

        prompt : typing.Optional[PostedConfigPromptSpec]

        timeouts : typing.Optional[PostedTimeoutSpecs]

        tools : typing.Optional[typing.Sequence[typing.Optional[PostedUserDefinedToolSpec]]]
            Tool specification for a Config.

        version_description : typing.Optional[str]
            An optional description of the Config version.

        voice : typing.Optional[VoiceRef]
            A voice specification associated with this Config.

        webhooks : typing.Optional[typing.Sequence[typing.Optional[PostedWebhookSpec]]]
            Webhook config specifications for each subscriber.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ReturnConfig]
            Created
        """
        _response = self._client_wrapper.httpx_client.request(
            "v0/evi/configs",
            base_url=self._client_wrapper.get_environment().base,
            method="POST",
            json={
                "builtin_tools": convert_and_respect_annotation_metadata(
                    object_=builtin_tools,
                    annotation=typing.Sequence[typing.Optional[PostedBuiltinTool]],
                    direction="write",
                ),
                "ellm_model": convert_and_respect_annotation_metadata(
                    object_=ellm_model, annotation=PostedEllmModel, direction="write"
                ),
                "event_messages": convert_and_respect_annotation_metadata(
                    object_=event_messages, annotation=PostedEventMessageSpecs, direction="write"
                ),
                "evi_version": evi_version,
                "language_model": convert_and_respect_annotation_metadata(
                    object_=language_model, annotation=PostedLanguageModel, direction="write"
                ),
                "name": name,
                "nudges": convert_and_respect_annotation_metadata(
                    object_=nudges, annotation=PostedNudgeSpec, direction="write"
                ),
                "prompt": convert_and_respect_annotation_metadata(
                    object_=prompt, annotation=PostedConfigPromptSpec, direction="write"
                ),
                "timeouts": convert_and_respect_annotation_metadata(
                    object_=timeouts, annotation=PostedTimeoutSpecs, direction="write"
                ),
                "tools": convert_and_respect_annotation_metadata(
                    object_=tools,
                    annotation=typing.Sequence[typing.Optional[PostedUserDefinedToolSpec]],
                    direction="write",
                ),
                "version_description": version_description,
                "voice": convert_and_respect_annotation_metadata(object_=voice, annotation=VoiceRef, direction="write"),
                "webhooks": convert_and_respect_annotation_metadata(
                    object_=webhooks, annotation=typing.Sequence[typing.Optional[PostedWebhookSpec]], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ReturnConfig,
                    parse_obj_as(
                        type_=ReturnConfig,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def list_config_versions(
        self,
        id: str,
        *,
        page_number: typing.Optional[int] = None,
        page_size: typing.Optional[int] = None,
        restrict_to_most_recent: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[ReturnConfig, ReturnPagedConfigs]:
        """
        Fetches a list of a **Config's** versions.

        For more details on configuration options and how to configure EVI, see our [configuration guide](/docs/speech-to-speech-evi/configuration).

        Parameters
        ----------
        id : str
            Identifier for a Config. Formatted as a UUID.

        page_number : typing.Optional[int]
            Specifies the page number to retrieve, enabling pagination.

            This parameter uses zero-based indexing. For example, setting `page_number` to 0 retrieves the first page of results (items 0-9 if `page_size` is 10), setting `page_number` to 1 retrieves the second page (items 10-19), and so on. Defaults to 0, which retrieves the first page.

        page_size : typing.Optional[int]
            Specifies the maximum number of results to include per page, enabling pagination. The value must be between 1 and 100, inclusive.

            For example, if `page_size` is set to 10, each page will include up to 10 items. Defaults to 10.

        restrict_to_most_recent : typing.Optional[bool]
            By default, `restrict_to_most_recent` is set to true, returning only the latest version of each config. To include all versions of each config in the list, set `restrict_to_most_recent` to false.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[ReturnConfig, ReturnPagedConfigs]
            Success
        """
        page_number = page_number if page_number is not None else 0

        _response = self._client_wrapper.httpx_client.request(
            f"v0/evi/configs/{jsonable_encoder(id)}",
            base_url=self._client_wrapper.get_environment().base,
            method="GET",
            params={
                "page_number": page_number,
                "page_size": page_size,
                "restrict_to_most_recent": restrict_to_most_recent,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ReturnPagedConfigs,
                    parse_obj_as(
                        type_=ReturnPagedConfigs,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.configs_page
                _has_next = True
                _get_next = lambda: self.list_config_versions(
                    id,
                    page_number=page_number + 1,
                    page_size=page_size,
                    restrict_to_most_recent=restrict_to_most_recent,
                    request_options=request_options,
                )
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create_config_version(
        self,
        id: str,
        *,
        evi_version: str,
        builtin_tools: typing.Optional[typing.Sequence[typing.Optional[PostedBuiltinTool]]] = OMIT,
        ellm_model: typing.Optional[PostedEllmModel] = OMIT,
        event_messages: typing.Optional[PostedEventMessageSpecs] = OMIT,
        language_model: typing.Optional[PostedLanguageModel] = OMIT,
        nudges: typing.Optional[PostedNudgeSpec] = OMIT,
        prompt: typing.Optional[PostedConfigPromptSpec] = OMIT,
        timeouts: typing.Optional[PostedTimeoutSpecs] = OMIT,
        tools: typing.Optional[typing.Sequence[typing.Optional[PostedUserDefinedToolSpec]]] = OMIT,
        version_description: typing.Optional[str] = OMIT,
        voice: typing.Optional[VoiceRef] = OMIT,
        webhooks: typing.Optional[typing.Sequence[typing.Optional[PostedWebhookSpec]]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ReturnConfig]:
        """
        Updates a **Config** by creating a new version of the **Config**.

        For more details on configuration options and how to configure EVI, see our [configuration guide](/docs/speech-to-speech-evi/configuration).

        Parameters
        ----------
        id : str
            Identifier for a Config. Formatted as a UUID.

        evi_version : str
            EVI version to use. Only versions `3` and `4-mini` are supported.

        builtin_tools : typing.Optional[typing.Sequence[typing.Optional[PostedBuiltinTool]]]
            Built-in tool specification for a Config.

        ellm_model : typing.Optional[PostedEllmModel]

        event_messages : typing.Optional[PostedEventMessageSpecs]

        language_model : typing.Optional[PostedLanguageModel]

        nudges : typing.Optional[PostedNudgeSpec]

        prompt : typing.Optional[PostedConfigPromptSpec]

        timeouts : typing.Optional[PostedTimeoutSpecs]

        tools : typing.Optional[typing.Sequence[typing.Optional[PostedUserDefinedToolSpec]]]
            Tool specification for a Config.

        version_description : typing.Optional[str]
            An optional description of the Config version.

        voice : typing.Optional[VoiceRef]

        webhooks : typing.Optional[typing.Sequence[typing.Optional[PostedWebhookSpec]]]
            Webhook config specifications for each subscriber.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ReturnConfig]
            Created
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v0/evi/configs/{jsonable_encoder(id)}",
            base_url=self._client_wrapper.get_environment().base,
            method="POST",
            json={
                "builtin_tools": convert_and_respect_annotation_metadata(
                    object_=builtin_tools,
                    annotation=typing.Sequence[typing.Optional[PostedBuiltinTool]],
                    direction="write",
                ),
                "ellm_model": convert_and_respect_annotation_metadata(
                    object_=ellm_model, annotation=PostedEllmModel, direction="write"
                ),
                "event_messages": convert_and_respect_annotation_metadata(
                    object_=event_messages, annotation=PostedEventMessageSpecs, direction="write"
                ),
                "evi_version": evi_version,
                "language_model": convert_and_respect_annotation_metadata(
                    object_=language_model, annotation=PostedLanguageModel, direction="write"
                ),
                "nudges": convert_and_respect_annotation_metadata(
                    object_=nudges, annotation=PostedNudgeSpec, direction="write"
                ),
                "prompt": convert_and_respect_annotation_metadata(
                    object_=prompt, annotation=PostedConfigPromptSpec, direction="write"
                ),
                "timeouts": convert_and_respect_annotation_metadata(
                    object_=timeouts, annotation=PostedTimeoutSpecs, direction="write"
                ),
                "tools": convert_and_respect_annotation_metadata(
                    object_=tools,
                    annotation=typing.Sequence[typing.Optional[PostedUserDefinedToolSpec]],
                    direction="write",
                ),
                "version_description": version_description,
                "voice": convert_and_respect_annotation_metadata(object_=voice, annotation=VoiceRef, direction="write"),
                "webhooks": convert_and_respect_annotation_metadata(
                    object_=webhooks, annotation=typing.Sequence[typing.Optional[PostedWebhookSpec]], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ReturnConfig,
                    parse_obj_as(
                        type_=ReturnConfig,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete_config(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> HttpResponse[None]:
        """
        Deletes a **Config** and its versions.

        For more details on configuration options and how to configure EVI, see our [configuration guide](/docs/speech-to-speech-evi/configuration).

        Parameters
        ----------
        id : str
            Identifier for a Config. Formatted as a UUID.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v0/evi/configs/{jsonable_encoder(id)}",
            base_url=self._client_wrapper.get_environment().base,
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update_config_name(
        self, id: str, *, name: str, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[str]:
        """
        Updates the name of a **Config**.

        For more details on configuration options and how to configure EVI, see our [configuration guide](/docs/speech-to-speech-evi/configuration).

        Parameters
        ----------
        id : str
            Identifier for a Config. Formatted as a UUID.

        name : str
            Name applied to all versions of a particular Config.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[str]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v0/evi/configs/{jsonable_encoder(id)}",
            base_url=self._client_wrapper.get_environment().base,
            method="PATCH",
            json={
                "name": name,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=_response.text)  # type: ignore
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_config_version(
        self, id: str, version: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[ReturnConfig]:
        """
        Fetches a specified version of a **Config**.

        For more details on configuration options and how to configure EVI, see our [configuration guide](/docs/speech-to-speech-evi/configuration).

        Parameters
        ----------
        id : str
            Identifier for a Config. Formatted as a UUID.

        version : int
            Version number for a Config.

            Configs, Prompts, Custom Voices, and Tools are versioned. This versioning system supports iterative development, allowing you to progressively refine configurations and revert to previous versions if needed.

            Version numbers are integer values representing different iterations of the Config. Each update to the Config increments its version number.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ReturnConfig]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v0/evi/configs/{jsonable_encoder(id)}/version/{jsonable_encoder(version)}",
            base_url=self._client_wrapper.get_environment().base,
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ReturnConfig,
                    parse_obj_as(
                        type_=ReturnConfig,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete_config_version(
        self, id: str, version: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[None]:
        """
        Deletes a specified version of a **Config**.

        For more details on configuration options and how to configure EVI, see our [configuration guide](/docs/speech-to-speech-evi/configuration).

        Parameters
        ----------
        id : str
            Identifier for a Config. Formatted as a UUID.

        version : int
            Version number for a Config.

            Configs, Prompts, Custom Voices, and Tools are versioned. This versioning system supports iterative development, allowing you to progressively refine configurations and revert to previous versions if needed.

            Version numbers are integer values representing different iterations of the Config. Each update to the Config increments its version number.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v0/evi/configs/{jsonable_encoder(id)}/version/{jsonable_encoder(version)}",
            base_url=self._client_wrapper.get_environment().base,
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update_config_description(
        self,
        id: str,
        version: int,
        *,
        version_description: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ReturnConfig]:
        """
        Updates the description of a **Config**.

        For more details on configuration options and how to configure EVI, see our [configuration guide](/docs/speech-to-speech-evi/configuration).

        Parameters
        ----------
        id : str
            Identifier for a Config. Formatted as a UUID.

        version : int
            Version number for a Config.

            Configs, Prompts, Custom Voices, and Tools are versioned. This versioning system supports iterative development, allowing you to progressively refine configurations and revert to previous versions if needed.

            Version numbers are integer values representing different iterations of the Config. Each update to the Config increments its version number.

        version_description : typing.Optional[str]
            An optional description of the Config version.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ReturnConfig]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v0/evi/configs/{jsonable_encoder(id)}/version/{jsonable_encoder(version)}",
            base_url=self._client_wrapper.get_environment().base,
            method="PATCH",
            json={
                "version_description": version_description,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ReturnConfig,
                    parse_obj_as(
                        type_=ReturnConfig,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawConfigsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list_configs(
        self,
        *,
        page_number: typing.Optional[int] = None,
        page_size: typing.Optional[int] = None,
        restrict_to_most_recent: typing.Optional[bool] = None,
        name: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[ReturnConfig, ReturnPagedConfigs]:
        """
        Fetches a paginated list of **Configs**.

        For more details on configuration options and how to configure EVI, see our [configuration guide](/docs/speech-to-speech-evi/configuration).

        Parameters
        ----------
        page_number : typing.Optional[int]
            Specifies the page number to retrieve, enabling pagination.

            This parameter uses zero-based indexing. For example, setting `page_number` to 0 retrieves the first page of results (items 0-9 if `page_size` is 10), setting `page_number` to 1 retrieves the second page (items 10-19), and so on. Defaults to 0, which retrieves the first page.

        page_size : typing.Optional[int]
            Specifies the maximum number of results to include per page, enabling pagination. The value must be between 1 and 100, inclusive.

            For example, if `page_size` is set to 10, each page will include up to 10 items. Defaults to 10.

        restrict_to_most_recent : typing.Optional[bool]
            By default, `restrict_to_most_recent` is set to true, returning only the latest version of each config. To include all versions of each config in the list, set `restrict_to_most_recent` to false.

        name : typing.Optional[str]
            Filter to only include configs with this name.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[ReturnConfig, ReturnPagedConfigs]
            Success
        """
        page_number = page_number if page_number is not None else 0

        _response = await self._client_wrapper.httpx_client.request(
            "v0/evi/configs",
            base_url=self._client_wrapper.get_environment().base,
            method="GET",
            params={
                "page_number": page_number,
                "page_size": page_size,
                "restrict_to_most_recent": restrict_to_most_recent,
                "name": name,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ReturnPagedConfigs,
                    parse_obj_as(
                        type_=ReturnPagedConfigs,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.configs_page
                _has_next = True

                async def _get_next():
                    return await self.list_configs(
                        page_number=page_number + 1,
                        page_size=page_size,
                        restrict_to_most_recent=restrict_to_most_recent,
                        name=name,
                        request_options=request_options,
                    )

                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create_config(
        self,
        *,
        evi_version: str,
        name: str,
        builtin_tools: typing.Optional[typing.Sequence[typing.Optional[PostedBuiltinTool]]] = OMIT,
        ellm_model: typing.Optional[PostedEllmModel] = OMIT,
        event_messages: typing.Optional[PostedEventMessageSpecs] = OMIT,
        language_model: typing.Optional[PostedLanguageModel] = OMIT,
        nudges: typing.Optional[PostedNudgeSpec] = OMIT,
        prompt: typing.Optional[PostedConfigPromptSpec] = OMIT,
        timeouts: typing.Optional[PostedTimeoutSpecs] = OMIT,
        tools: typing.Optional[typing.Sequence[typing.Optional[PostedUserDefinedToolSpec]]] = OMIT,
        version_description: typing.Optional[str] = OMIT,
        voice: typing.Optional[VoiceRef] = OMIT,
        webhooks: typing.Optional[typing.Sequence[typing.Optional[PostedWebhookSpec]]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ReturnConfig]:
        """
        Creates a **Config** which can be applied to EVI.

        For more details on configuration options and how to configure EVI, see our [configuration guide](/docs/speech-to-speech-evi/configuration).

        Parameters
        ----------
        evi_version : str
            EVI version to use. Only versions `3` and `4-mini` are supported.

        name : str
            Name applied to all versions of a particular Config.

        builtin_tools : typing.Optional[typing.Sequence[typing.Optional[PostedBuiltinTool]]]
            Built-in tool specification for a Config.

        ellm_model : typing.Optional[PostedEllmModel]

        event_messages : typing.Optional[PostedEventMessageSpecs]

        language_model : typing.Optional[PostedLanguageModel]

        nudges : typing.Optional[PostedNudgeSpec]

        prompt : typing.Optional[PostedConfigPromptSpec]

        timeouts : typing.Optional[PostedTimeoutSpecs]

        tools : typing.Optional[typing.Sequence[typing.Optional[PostedUserDefinedToolSpec]]]
            Tool specification for a Config.

        version_description : typing.Optional[str]
            An optional description of the Config version.

        voice : typing.Optional[VoiceRef]
            A voice specification associated with this Config.

        webhooks : typing.Optional[typing.Sequence[typing.Optional[PostedWebhookSpec]]]
            Webhook config specifications for each subscriber.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ReturnConfig]
            Created
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v0/evi/configs",
            base_url=self._client_wrapper.get_environment().base,
            method="POST",
            json={
                "builtin_tools": convert_and_respect_annotation_metadata(
                    object_=builtin_tools,
                    annotation=typing.Sequence[typing.Optional[PostedBuiltinTool]],
                    direction="write",
                ),
                "ellm_model": convert_and_respect_annotation_metadata(
                    object_=ellm_model, annotation=PostedEllmModel, direction="write"
                ),
                "event_messages": convert_and_respect_annotation_metadata(
                    object_=event_messages, annotation=PostedEventMessageSpecs, direction="write"
                ),
                "evi_version": evi_version,
                "language_model": convert_and_respect_annotation_metadata(
                    object_=language_model, annotation=PostedLanguageModel, direction="write"
                ),
                "name": name,
                "nudges": convert_and_respect_annotation_metadata(
                    object_=nudges, annotation=PostedNudgeSpec, direction="write"
                ),
                "prompt": convert_and_respect_annotation_metadata(
                    object_=prompt, annotation=PostedConfigPromptSpec, direction="write"
                ),
                "timeouts": convert_and_respect_annotation_metadata(
                    object_=timeouts, annotation=PostedTimeoutSpecs, direction="write"
                ),
                "tools": convert_and_respect_annotation_metadata(
                    object_=tools,
                    annotation=typing.Sequence[typing.Optional[PostedUserDefinedToolSpec]],
                    direction="write",
                ),
                "version_description": version_description,
                "voice": convert_and_respect_annotation_metadata(object_=voice, annotation=VoiceRef, direction="write"),
                "webhooks": convert_and_respect_annotation_metadata(
                    object_=webhooks, annotation=typing.Sequence[typing.Optional[PostedWebhookSpec]], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ReturnConfig,
                    parse_obj_as(
                        type_=ReturnConfig,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def list_config_versions(
        self,
        id: str,
        *,
        page_number: typing.Optional[int] = None,
        page_size: typing.Optional[int] = None,
        restrict_to_most_recent: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[ReturnConfig, ReturnPagedConfigs]:
        """
        Fetches a list of a **Config's** versions.

        For more details on configuration options and how to configure EVI, see our [configuration guide](/docs/speech-to-speech-evi/configuration).

        Parameters
        ----------
        id : str
            Identifier for a Config. Formatted as a UUID.

        page_number : typing.Optional[int]
            Specifies the page number to retrieve, enabling pagination.

            This parameter uses zero-based indexing. For example, setting `page_number` to 0 retrieves the first page of results (items 0-9 if `page_size` is 10), setting `page_number` to 1 retrieves the second page (items 10-19), and so on. Defaults to 0, which retrieves the first page.

        page_size : typing.Optional[int]
            Specifies the maximum number of results to include per page, enabling pagination. The value must be between 1 and 100, inclusive.

            For example, if `page_size` is set to 10, each page will include up to 10 items. Defaults to 10.

        restrict_to_most_recent : typing.Optional[bool]
            By default, `restrict_to_most_recent` is set to true, returning only the latest version of each config. To include all versions of each config in the list, set `restrict_to_most_recent` to false.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[ReturnConfig, ReturnPagedConfigs]
            Success
        """
        page_number = page_number if page_number is not None else 0

        _response = await self._client_wrapper.httpx_client.request(
            f"v0/evi/configs/{jsonable_encoder(id)}",
            base_url=self._client_wrapper.get_environment().base,
            method="GET",
            params={
                "page_number": page_number,
                "page_size": page_size,
                "restrict_to_most_recent": restrict_to_most_recent,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ReturnPagedConfigs,
                    parse_obj_as(
                        type_=ReturnPagedConfigs,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.configs_page
                _has_next = True

                async def _get_next():
                    return await self.list_config_versions(
                        id,
                        page_number=page_number + 1,
                        page_size=page_size,
                        restrict_to_most_recent=restrict_to_most_recent,
                        request_options=request_options,
                    )

                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create_config_version(
        self,
        id: str,
        *,
        evi_version: str,
        builtin_tools: typing.Optional[typing.Sequence[typing.Optional[PostedBuiltinTool]]] = OMIT,
        ellm_model: typing.Optional[PostedEllmModel] = OMIT,
        event_messages: typing.Optional[PostedEventMessageSpecs] = OMIT,
        language_model: typing.Optional[PostedLanguageModel] = OMIT,
        nudges: typing.Optional[PostedNudgeSpec] = OMIT,
        prompt: typing.Optional[PostedConfigPromptSpec] = OMIT,
        timeouts: typing.Optional[PostedTimeoutSpecs] = OMIT,
        tools: typing.Optional[typing.Sequence[typing.Optional[PostedUserDefinedToolSpec]]] = OMIT,
        version_description: typing.Optional[str] = OMIT,
        voice: typing.Optional[VoiceRef] = OMIT,
        webhooks: typing.Optional[typing.Sequence[typing.Optional[PostedWebhookSpec]]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ReturnConfig]:
        """
        Updates a **Config** by creating a new version of the **Config**.

        For more details on configuration options and how to configure EVI, see our [configuration guide](/docs/speech-to-speech-evi/configuration).

        Parameters
        ----------
        id : str
            Identifier for a Config. Formatted as a UUID.

        evi_version : str
            EVI version to use. Only versions `3` and `4-mini` are supported.

        builtin_tools : typing.Optional[typing.Sequence[typing.Optional[PostedBuiltinTool]]]
            Built-in tool specification for a Config.

        ellm_model : typing.Optional[PostedEllmModel]

        event_messages : typing.Optional[PostedEventMessageSpecs]

        language_model : typing.Optional[PostedLanguageModel]

        nudges : typing.Optional[PostedNudgeSpec]

        prompt : typing.Optional[PostedConfigPromptSpec]

        timeouts : typing.Optional[PostedTimeoutSpecs]

        tools : typing.Optional[typing.Sequence[typing.Optional[PostedUserDefinedToolSpec]]]
            Tool specification for a Config.

        version_description : typing.Optional[str]
            An optional description of the Config version.

        voice : typing.Optional[VoiceRef]

        webhooks : typing.Optional[typing.Sequence[typing.Optional[PostedWebhookSpec]]]
            Webhook config specifications for each subscriber.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ReturnConfig]
            Created
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v0/evi/configs/{jsonable_encoder(id)}",
            base_url=self._client_wrapper.get_environment().base,
            method="POST",
            json={
                "builtin_tools": convert_and_respect_annotation_metadata(
                    object_=builtin_tools,
                    annotation=typing.Sequence[typing.Optional[PostedBuiltinTool]],
                    direction="write",
                ),
                "ellm_model": convert_and_respect_annotation_metadata(
                    object_=ellm_model, annotation=PostedEllmModel, direction="write"
                ),
                "event_messages": convert_and_respect_annotation_metadata(
                    object_=event_messages, annotation=PostedEventMessageSpecs, direction="write"
                ),
                "evi_version": evi_version,
                "language_model": convert_and_respect_annotation_metadata(
                    object_=language_model, annotation=PostedLanguageModel, direction="write"
                ),
                "nudges": convert_and_respect_annotation_metadata(
                    object_=nudges, annotation=PostedNudgeSpec, direction="write"
                ),
                "prompt": convert_and_respect_annotation_metadata(
                    object_=prompt, annotation=PostedConfigPromptSpec, direction="write"
                ),
                "timeouts": convert_and_respect_annotation_metadata(
                    object_=timeouts, annotation=PostedTimeoutSpecs, direction="write"
                ),
                "tools": convert_and_respect_annotation_metadata(
                    object_=tools,
                    annotation=typing.Sequence[typing.Optional[PostedUserDefinedToolSpec]],
                    direction="write",
                ),
                "version_description": version_description,
                "voice": convert_and_respect_annotation_metadata(object_=voice, annotation=VoiceRef, direction="write"),
                "webhooks": convert_and_respect_annotation_metadata(
                    object_=webhooks, annotation=typing.Sequence[typing.Optional[PostedWebhookSpec]], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ReturnConfig,
                    parse_obj_as(
                        type_=ReturnConfig,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete_config(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        Deletes a **Config** and its versions.

        For more details on configuration options and how to configure EVI, see our [configuration guide](/docs/speech-to-speech-evi/configuration).

        Parameters
        ----------
        id : str
            Identifier for a Config. Formatted as a UUID.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v0/evi/configs/{jsonable_encoder(id)}",
            base_url=self._client_wrapper.get_environment().base,
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update_config_name(
        self, id: str, *, name: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[str]:
        """
        Updates the name of a **Config**.

        For more details on configuration options and how to configure EVI, see our [configuration guide](/docs/speech-to-speech-evi/configuration).

        Parameters
        ----------
        id : str
            Identifier for a Config. Formatted as a UUID.

        name : str
            Name applied to all versions of a particular Config.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[str]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v0/evi/configs/{jsonable_encoder(id)}",
            base_url=self._client_wrapper.get_environment().base,
            method="PATCH",
            json={
                "name": name,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=_response.text)  # type: ignore
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_config_version(
        self, id: str, version: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[ReturnConfig]:
        """
        Fetches a specified version of a **Config**.

        For more details on configuration options and how to configure EVI, see our [configuration guide](/docs/speech-to-speech-evi/configuration).

        Parameters
        ----------
        id : str
            Identifier for a Config. Formatted as a UUID.

        version : int
            Version number for a Config.

            Configs, Prompts, Custom Voices, and Tools are versioned. This versioning system supports iterative development, allowing you to progressively refine configurations and revert to previous versions if needed.

            Version numbers are integer values representing different iterations of the Config. Each update to the Config increments its version number.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ReturnConfig]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v0/evi/configs/{jsonable_encoder(id)}/version/{jsonable_encoder(version)}",
            base_url=self._client_wrapper.get_environment().base,
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ReturnConfig,
                    parse_obj_as(
                        type_=ReturnConfig,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete_config_version(
        self, id: str, version: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        Deletes a specified version of a **Config**.

        For more details on configuration options and how to configure EVI, see our [configuration guide](/docs/speech-to-speech-evi/configuration).

        Parameters
        ----------
        id : str
            Identifier for a Config. Formatted as a UUID.

        version : int
            Version number for a Config.

            Configs, Prompts, Custom Voices, and Tools are versioned. This versioning system supports iterative development, allowing you to progressively refine configurations and revert to previous versions if needed.

            Version numbers are integer values representing different iterations of the Config. Each update to the Config increments its version number.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v0/evi/configs/{jsonable_encoder(id)}/version/{jsonable_encoder(version)}",
            base_url=self._client_wrapper.get_environment().base,
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update_config_description(
        self,
        id: str,
        version: int,
        *,
        version_description: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ReturnConfig]:
        """
        Updates the description of a **Config**.

        For more details on configuration options and how to configure EVI, see our [configuration guide](/docs/speech-to-speech-evi/configuration).

        Parameters
        ----------
        id : str
            Identifier for a Config. Formatted as a UUID.

        version : int
            Version number for a Config.

            Configs, Prompts, Custom Voices, and Tools are versioned. This versioning system supports iterative development, allowing you to progressively refine configurations and revert to previous versions if needed.

            Version numbers are integer values representing different iterations of the Config. Each update to the Config increments its version number.

        version_description : typing.Optional[str]
            An optional description of the Config version.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ReturnConfig]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v0/evi/configs/{jsonable_encoder(id)}/version/{jsonable_encoder(version)}",
            base_url=self._client_wrapper.get_environment().base,
            method="PATCH",
            json={
                "version_description": version_description,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ReturnConfig,
                    parse_obj_as(
                        type_=ReturnConfig,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .types import (
        AssistantEnd,
        AssistantInput,
        AssistantMessage,
        AssistantProsody,
        AudioConfiguration,
        AudioInput,
        AudioOutput,
        BuiltInTool,
        BuiltinToolConfig,
        ChatMessage,
        ChatMessageToolResult,
        ChatMetadata,
        ConnectSessionSettings,
        ConnectSessionSettingsAudio,
        ConnectSessionSettingsContext,
        ConnectSessionSettingsVariablesValue,
        Context,
        ContextType,
        ControlPlanePublishEvent,
        EmotionScores,
        Encoding,
        ErrorLevel,
        ErrorResponse,
        HttpValidationError,
        Inference,
        JsonMessage,
        LanguageModelType,
        MillisecondInterval,
        ModelProviderEnum,
        PauseAssistantMessage,
        PostedBuiltinTool,
        PostedBuiltinToolName,
        PostedConfigPromptSpec,
        PostedEllmModel,
        PostedEventMessageSpec,
        PostedEventMessageSpecs,
        PostedLanguageModel,
        PostedNudgeSpec,
        PostedTimeoutSpec,
        PostedTimeoutSpecs,
        PostedTimeoutSpecsInactivity,
        PostedTimeoutSpecsMaxDuration,
        PostedUserDefinedToolSpec,
        PostedWebhookEventType,
        PostedWebhookSpec,
        ProsodyInference,
        ResumeAssistantMessage,
        ReturnBuiltinTool,
        ReturnBuiltinToolToolType,
        ReturnChat,
        ReturnChatAudioReconstruction,
        ReturnChatAudioReconstructionStatus,
        ReturnChatEvent,
        ReturnChatEventRole,
        ReturnChatEventType,
        ReturnChatGroup,
        ReturnChatGroupPagedAudioReconstructions,
        ReturnChatGroupPagedAudioReconstructionsPaginationDirection,
        ReturnChatGroupPagedChats,
        ReturnChatGroupPagedChatsPaginationDirection,
        ReturnChatGroupPagedEvents,
        ReturnChatGroupPagedEventsPaginationDirection,
        ReturnChatPagedEvents,
        ReturnChatPagedEventsPaginationDirection,
        ReturnChatPagedEventsStatus,
        ReturnChatStatus,
        ReturnConfig,
        ReturnConfigSpec,
        ReturnEllmModel,
        ReturnEventMessageSpec,
        ReturnEventMessageSpecs,
        ReturnLanguageModel,
        ReturnNudgeSpec,
        ReturnPagedChatGroups,
        ReturnPagedChatGroupsPaginationDirection,
        ReturnPagedChats,
        ReturnPagedChatsPaginationDirection,
        ReturnPagedConfigs,
        ReturnPagedPrompts,
        ReturnPagedUserDefinedTools,
        ReturnPrompt,
        ReturnPromptVersionType,
        ReturnTimeoutSpec,
        ReturnTimeoutSpecs,
        ReturnUserDefinedTool,
        ReturnUserDefinedToolToolType,
        ReturnUserDefinedToolVersionType,
        ReturnVoice,
        ReturnWebhookEventType,
        ReturnWebhookSpec,
        Role,
        SessionSettings,
        SessionSettingsVariablesValue,
        SubscribeEvent,
        Tool,
        ToolCallMessage,
        ToolErrorMessage,
        ToolResponseMessage,
        ToolType,
        UserInput,
        UserInterruption,
        UserMessage,
        ValidationError,
        ValidationErrorLocItem,
        VoiceId,
        VoiceName,
        VoiceProvider,
        VoiceRef,
        WebSocketError,
        WebhookEvent,
        WebhookEventBase,
        WebhookEventChatEnded,
        WebhookEventChatStartType,
        WebhookEventChatStarted,
        WebhookEventChatStatus,
        WebhookEventToolCall,
    )
    from .errors import BadRequestError, UnprocessableEntityError
    from . import chat, chat_groups, chats, configs, control_plane, prompts, tools
    from .chat import PublishEvent
_dynamic_imports: typing.Dict[str, str] = {
    "AssistantEnd": ".types",
    "AssistantInput": ".types",
    "AssistantMessage": ".types",
    "AssistantProsody": ".types",
    "AudioConfiguration": ".types",
    "AudioInput": ".types",
    "AudioOutput": ".types",
    "BadRequestError": ".errors",
    "BuiltInTool": ".types",
    "BuiltinToolConfig": ".types",
    "ChatMessage": ".types",
    "ChatMessageToolResult": ".types",
    "ChatMetadata": ".types",
    "ConnectSessionSettings": ".types",
    "ConnectSessionSettingsAudio": ".types",
    "ConnectSessionSettingsContext": ".types",
    "ConnectSessionSettingsVariablesValue": ".types",
    "Context": ".types",
    "ContextType": ".types",
    "ControlPlanePublishEvent": ".types",
    "EmotionScores": ".types",
    "Encoding": ".types",
    "ErrorLevel": ".types",
    "ErrorResponse": ".types",
    "HttpValidationError": ".types",
    "Inference": ".types",
    "JsonMessage": ".types",
    "LanguageModelType": ".types",
    "MillisecondInterval": ".types",
    "ModelProviderEnum": ".types",
    "PauseAssistantMessage": ".types",
    "PostedBuiltinTool": ".types",
    "PostedBuiltinToolName": ".types",
    "PostedConfigPromptSpec": ".types",
    "PostedEllmModel": ".types",
    "PostedEventMessageSpec": ".types",
    "PostedEventMessageSpecs": ".types",
    "PostedLanguageModel": ".types",
    "PostedNudgeSpec": ".types",
    "PostedTimeoutSpec": ".types",
    "PostedTimeoutSpecs": ".types",
    "PostedTimeoutSpecsInactivity": ".types",
    "PostedTimeoutSpecsMaxDuration": ".types",
    "PostedUserDefinedToolSpec": ".types",
    "PostedWebhookEventType": ".types",
    "PostedWebhookSpec": ".types",
    "ProsodyInference": ".types",
    "PublishEvent": ".chat",
    "ResumeAssistantMessage": ".types",
    "ReturnBuiltinTool": ".types",
    "ReturnBuiltinToolToolType": ".types",
    "ReturnChat": ".types",
    "ReturnChatAudioReconstruction": ".types",
    "ReturnChatAudioReconstructionStatus": ".types",
    "ReturnChatEvent": ".types",
    "ReturnChatEventRole": ".types",
    "ReturnChatEventType": ".types",
    "ReturnChatGroup": ".types",
    "ReturnChatGroupPagedAudioReconstructions": ".types",
    "ReturnChatGroupPagedAudioReconstructionsPaginationDirection": ".types",
    "ReturnChatGroupPagedChats": ".types",
    "ReturnChatGroupPagedChatsPaginationDirection": ".types",
    "ReturnChatGroupPagedEvents": ".types",
    "ReturnChatGroupPagedEventsPaginationDirection": ".types",
    "ReturnChatPagedEvents": ".types",
    "ReturnChatPagedEventsPaginationDirection": ".types",
    "ReturnChatPagedEventsStatus": ".types",
    "ReturnChatStatus": ".types",
    "ReturnConfig": ".types",
    "ReturnConfigSpec": ".types",
    "ReturnEllmModel": ".types",
    "ReturnEventMessageSpec": ".types",
    "ReturnEventMessageSpecs": ".types",
    "ReturnLanguageModel": ".types",
    "ReturnNudgeSpec": ".types",
    "ReturnPagedChatGroups": ".types",
    "ReturnPagedChatGroupsPaginationDirection": ".types",
    "ReturnPagedChats": ".types",
    "ReturnPagedChatsPaginationDirection": ".types",
    "ReturnPagedConfigs": ".types",
    "ReturnPagedPrompts": ".types",
    "ReturnPagedUserDefinedTools": ".types",
    "ReturnPrompt": ".types",
    "ReturnPromptVersionType": ".types",
    "ReturnTimeoutSpec": ".types",
    "ReturnTimeoutSpecs": ".types",
    "ReturnUserDefinedTool": ".types",
    "ReturnUserDefinedToolToolType": ".types",
    "ReturnUserDefinedToolVersionType": ".types",
    "ReturnVoice": ".types",
    "ReturnWebhookEventType": ".types",
    "ReturnWebhookSpec": ".types",
    "Role": ".types",
    "SessionSettings": ".types",
    "SessionSettingsVariablesValue": ".types",
    "SubscribeEvent": ".types",
    "Tool": ".types",
    "ToolCallMessage": ".types",
    "ToolErrorMessage": ".types",
    "ToolResponseMessage": ".types",
    "ToolType": ".types",
    "UnprocessableEntityError": ".errors",
    "UserInput": ".types",
    "UserInterruption": ".types",
    "UserMessage": ".types",
    "ValidationError": ".types",
    "ValidationErrorLocItem": ".types",
    "VoiceId": ".types",
    "VoiceName": ".types",
    "VoiceProvider": ".types",
    "VoiceRef": ".types",
    "WebSocketError": ".types",
    "WebhookEvent": ".types",
    "WebhookEventBase": ".types",
    "WebhookEventChatEnded": ".types",
    "WebhookEventChatStartType": ".types",
    "WebhookEventChatStarted": ".types",
    "WebhookEventChatStatus": ".types",
    "WebhookEventToolCall": ".types",
    "chat": ".chat",
    "chat_groups": ".chat_groups",
    "chats": ".chats",
    "configs": ".configs",
    "control_plane": ".control_plane",
    "prompts": ".prompts",
    "tools": ".tools",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "AssistantEnd",
    "AssistantInput",
    "AssistantMessage",
    "AssistantProsody",
    "AudioConfiguration",
    "AudioInput",
    "AudioOutput",
    "BadRequestError",
    "BuiltInTool",
    "BuiltinToolConfig",
    "ChatMessage",
    "ChatMessageToolResult",
    "ChatMetadata",
    "ConnectSessionSettings",
    "ConnectSessionSettingsAudio",
    "ConnectSessionSettingsContext",
    "ConnectSessionSettingsVariablesValue",
    "Context",
    "ContextType",
    "ControlPlanePublishEvent",
    "EmotionScores",
    "Encoding",
    "ErrorLevel",
    "ErrorResponse",
    "HttpValidationError",
    "Inference",
    "JsonMessage",
    "LanguageModelType",
    "MillisecondInterval",
    "ModelProviderEnum",
    "PauseAssistantMessage",
    "PostedBuiltinTool",
    "PostedBuiltinToolName",
    "PostedConfigPromptSpec",
    "PostedEllmModel",
    "PostedEventMessageSpec",
    "PostedEventMessageSpecs",
    "PostedLanguageModel",
    "PostedNudgeSpec",
    "PostedTimeoutSpec",
    "PostedTimeoutSpecs",
    "PostedTimeoutSpecsInactivity",
    "PostedTimeoutSpecsMaxDuration",
    "PostedUserDefinedToolSpec",
    "PostedWebhookEventType",
    "PostedWebhookSpec",
    "ProsodyInference",
    "PublishEvent",
    "ResumeAssistantMessage",
    "ReturnBuiltinTool",
    "ReturnBuiltinToolToolType",
    "ReturnChat",
    "ReturnChatAudioReconstruction",
    "ReturnChatAudioReconstructionStatus",
    "ReturnChatEvent",
    "ReturnChatEventRole",
    "ReturnChatEventType",
    "ReturnChatGroup",
    "ReturnChatGroupPagedAudioReconstructions",
    "ReturnChatGroupPagedAudioReconstructionsPaginationDirection",
    "ReturnChatGroupPagedChats",
    "ReturnChatGroupPagedChatsPaginationDirection",
    "ReturnChatGroupPagedEvents",
    "ReturnChatGroupPagedEventsPaginationDirection",
    "ReturnChatPagedEvents",
    "ReturnChatPagedEventsPaginationDirection",
    "ReturnChatPagedEventsStatus",
    "ReturnChatStatus",
    "ReturnConfig",
    "ReturnConfigSpec",
    "ReturnEllmModel",
    "ReturnEventMessageSpec",
    "ReturnEventMessageSpecs",
    "ReturnLanguageModel",
    "ReturnNudgeSpec",
    "ReturnPagedChatGroups",
    "ReturnPagedChatGroupsPaginationDirection",
    "ReturnPagedChats",
    "ReturnPagedChatsPaginationDirection",
    "ReturnPagedConfigs",
    "ReturnPagedPrompts",
    "ReturnPagedUserDefinedTools",
    "ReturnPrompt",
    "ReturnPromptVersionType",
    "ReturnTimeoutSpec",
    "ReturnTimeoutSpecs",
    "ReturnUserDefinedTool",
    "ReturnUserDefinedToolToolType",
    "ReturnUserDefinedToolVersionType",
    "ReturnVoice",
    "ReturnWebhookEventType",
    "ReturnWebhookSpec",
    "Role",
    "SessionSettings",
    "SessionSettingsVariablesValue",
    "SubscribeEvent",
    "Tool",
    "ToolCallMessage",
    "ToolErrorMessage",
    "ToolResponseMessage",
    "ToolType",
    "UnprocessableEntityError",
    "UserInput",
    "UserInterruption",
    "UserMessage",
    "ValidationError",
    "ValidationErrorLocItem",
    "VoiceId",
    "VoiceName",
    "VoiceProvider",
    "VoiceRef",
    "WebSocketError",
    "WebhookEvent",
    "WebhookEventBase",
    "WebhookEventChatEnded",
    "WebhookEventChatStartType",
    "WebhookEventChatStarted",
    "WebhookEventChatStatus",
    "WebhookEventToolCall",
    "chat",
    "chat_groups",
    "chats",
    "configs",
    "control_plane",
    "prompts",
    "tools",
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
import typing_extensions
from ....core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class StateInference_Queued(UniversalBaseModel):
    status: typing.Literal["QUEUED"] = "QUEUED"
    created_timestamp_ms: int

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class StateInference_InProgress(UniversalBaseModel):
    status: typing.Literal["IN_PROGRESS"] = "IN_PROGRESS"
    created_timestamp_ms: int
    started_timestamp_ms: int

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class StateInference_Completed(UniversalBaseModel):
    status: typing.Literal["COMPLETED"] = "COMPLETED"
    created_timestamp_ms: int
    started_timestamp_ms: int
    ended_timestamp_ms: int
    num_predictions: int
    num_errors: int

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class StateInference_Failed(UniversalBaseModel):
    status: typing.Literal["FAILED"] = "FAILED"
    created_timestamp_ms: int
    started_timestamp_ms: int
    ended_timestamp_ms: int
    message: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


StateInference = typing_extensions.Annotated[
    typing.Union[StateInference_Queued, StateInference_InProgress, StateInference_Completed, StateInference_Failed],
    pydantic.Field(discriminator="status"),
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .assistant_end import AssistantEnd
    from .assistant_input import AssistantInput
    from .assistant_message import AssistantMessage
    from .assistant_prosody import AssistantProsody
    from .audio_configuration import AudioConfiguration
    from .audio_input import AudioInput
    from .audio_output import AudioOutput
    from .built_in_tool import BuiltInTool
    from .builtin_tool_config import BuiltinToolConfig
    from .chat_message import ChatMessage
    from .chat_message_tool_result import ChatMessageToolResult
    from .chat_metadata import ChatMetadata
    from .connect_session_settings import ConnectSessionSettings
    from .connect_session_settings_audio import ConnectSessionSettingsAudio
    from .connect_session_settings_context import ConnectSessionSettingsContext
    from .connect_session_settings_variables_value import ConnectSessionSettingsVariablesValue
    from .context import Context
    from .context_type import ContextType
    from .control_plane_publish_event import ControlPlanePublishEvent
    from .emotion_scores import EmotionScores
    from .encoding import Encoding
    from .error_level import ErrorLevel
    from .error_response import ErrorResponse
    from .http_validation_error import HttpValidationError
    from .inference import Inference
    from .json_message import JsonMessage
    from .language_model_type import LanguageModelType
    from .millisecond_interval import MillisecondInterval
    from .model_provider_enum import ModelProviderEnum
    from .pause_assistant_message import PauseAssistantMessage
    from .posted_builtin_tool import PostedBuiltinTool
    from .posted_builtin_tool_name import PostedBuiltinToolName
    from .posted_config_prompt_spec import PostedConfigPromptSpec
    from .posted_ellm_model import PostedEllmModel
    from .posted_event_message_spec import PostedEventMessageSpec
    from .posted_event_message_specs import PostedEventMessageSpecs
    from .posted_language_model import PostedLanguageModel
    from .posted_nudge_spec import PostedNudgeSpec
    from .posted_timeout_spec import PostedTimeoutSpec
    from .posted_timeout_specs import PostedTimeoutSpecs
    from .posted_timeout_specs_inactivity import PostedTimeoutSpecsInactivity
    from .posted_timeout_specs_max_duration import PostedTimeoutSpecsMaxDuration
    from .posted_user_defined_tool_spec import PostedUserDefinedToolSpec
    from .posted_webhook_event_type import PostedWebhookEventType
    from .posted_webhook_spec import PostedWebhookSpec
    from .prosody_inference import ProsodyInference
    from .resume_assistant_message import ResumeAssistantMessage
    from .return_builtin_tool import ReturnBuiltinTool
    from .return_builtin_tool_tool_type import ReturnBuiltinToolToolType
    from .return_chat import ReturnChat
    from .return_chat_audio_reconstruction import ReturnChatAudioReconstruction
    from .return_chat_audio_reconstruction_status import ReturnChatAudioReconstructionStatus
    from .return_chat_event import ReturnChatEvent
    from .return_chat_event_role import ReturnChatEventRole
    from .return_chat_event_type import ReturnChatEventType
    from .return_chat_group import ReturnChatGroup
    from .return_chat_group_paged_audio_reconstructions import ReturnChatGroupPagedAudioReconstructions
    from .return_chat_group_paged_audio_reconstructions_pagination_direction import (
        ReturnChatGroupPagedAudioReconstructionsPaginationDirection,
    )
    from .return_chat_group_paged_chats import ReturnChatGroupPagedChats
    from .return_chat_group_paged_chats_pagination_direction import ReturnChatGroupPagedChatsPaginationDirection
    from .return_chat_group_paged_events import ReturnChatGroupPagedEvents
    from .return_chat_group_paged_events_pagination_direction import ReturnChatGroupPagedEventsPaginationDirection
    from .return_chat_paged_events import ReturnChatPagedEvents
    from .return_chat_paged_events_pagination_direction import ReturnChatPagedEventsPaginationDirection
    from .return_chat_paged_events_status import ReturnChatPagedEventsStatus
    from .return_chat_status import ReturnChatStatus
    from .return_config import ReturnConfig
    from .return_config_spec import ReturnConfigSpec
    from .return_ellm_model import ReturnEllmModel
    from .return_event_message_spec import ReturnEventMessageSpec
    from .return_event_message_specs import ReturnEventMessageSpecs
    from .return_language_model import ReturnLanguageModel
    from .return_nudge_spec import ReturnNudgeSpec
    from .return_paged_chat_groups import ReturnPagedChatGroups
    from .return_paged_chat_groups_pagination_direction import ReturnPagedChatGroupsPaginationDirection
    from .return_paged_chats import ReturnPagedChats
    from .return_paged_chats_pagination_direction import ReturnPagedChatsPaginationDirection
    from .return_paged_configs import ReturnPagedConfigs
    from .return_paged_prompts import ReturnPagedPrompts
    from .return_paged_user_defined_tools import ReturnPagedUserDefinedTools
    from .return_prompt import ReturnPrompt
    from .return_prompt_version_type import ReturnPromptVersionType
    from .return_timeout_spec import ReturnTimeoutSpec
    from .return_timeout_specs import ReturnTimeoutSpecs
    from .return_user_defined_tool import ReturnUserDefinedTool
    from .return_user_defined_tool_tool_type import ReturnUserDefinedToolToolType
    from .return_user_defined_tool_version_type import ReturnUserDefinedToolVersionType
    from .return_voice import ReturnVoice
    from .return_webhook_event_type import ReturnWebhookEventType
    from .return_webhook_spec import ReturnWebhookSpec
    from .role import Role
    from .session_settings import SessionSettings
    from .session_settings_variables_value import SessionSettingsVariablesValue
    from .subscribe_event import SubscribeEvent
    from .tool import Tool
    from .tool_call_message import ToolCallMessage
    from .tool_error_message import ToolErrorMessage
    from .tool_response_message import ToolResponseMessage
    from .tool_type import ToolType
    from .user_input import UserInput
    from .user_interruption import UserInterruption
    from .user_message import UserMessage
    from .validation_error import ValidationError
    from .validation_error_loc_item import ValidationErrorLocItem
    from .voice_id import VoiceId
    from .voice_name import VoiceName
    from .voice_provider import VoiceProvider
    from .voice_ref import VoiceRef
    from .web_socket_error import WebSocketError
    from .webhook_event import WebhookEvent
    from .webhook_event_base import WebhookEventBase
    from .webhook_event_chat_ended import WebhookEventChatEnded
    from .webhook_event_chat_start_type import WebhookEventChatStartType
    from .webhook_event_chat_started import WebhookEventChatStarted
    from .webhook_event_chat_status import WebhookEventChatStatus
    from .webhook_event_tool_call import WebhookEventToolCall
_dynamic_imports: typing.Dict[str, str] = {
    "AssistantEnd": ".assistant_end",
    "AssistantInput": ".assistant_input",
    "AssistantMessage": ".assistant_message",
    "AssistantProsody": ".assistant_prosody",
    "AudioConfiguration": ".audio_configuration",
    "AudioInput": ".audio_input",
    "AudioOutput": ".audio_output",
    "BuiltInTool": ".built_in_tool",
    "BuiltinToolConfig": ".builtin_tool_config",
    "ChatMessage": ".chat_message",
    "ChatMessageToolResult": ".chat_message_tool_result",
    "ChatMetadata": ".chat_metadata",
    "ConnectSessionSettings": ".connect_session_settings",
    "ConnectSessionSettingsAudio": ".connect_session_settings_audio",
    "ConnectSessionSettingsContext": ".connect_session_settings_context",
    "ConnectSessionSettingsVariablesValue": ".connect_session_settings_variables_value",
    "Context": ".context",
    "ContextType": ".context_type",
    "ControlPlanePublishEvent": ".control_plane_publish_event",
    "EmotionScores": ".emotion_scores",
    "Encoding": ".encoding",
    "ErrorLevel": ".error_level",
    "ErrorResponse": ".error_response",
    "HttpValidationError": ".http_validation_error",
    "Inference": ".inference",
    "JsonMessage": ".json_message",
    "LanguageModelType": ".language_model_type",
    "MillisecondInterval": ".millisecond_interval",
    "ModelProviderEnum": ".model_provider_enum",
    "PauseAssistantMessage": ".pause_assistant_message",
    "PostedBuiltinTool": ".posted_builtin_tool",
    "PostedBuiltinToolName": ".posted_builtin_tool_name",
    "PostedConfigPromptSpec": ".posted_config_prompt_spec",
    "PostedEllmModel": ".posted_ellm_model",
    "PostedEventMessageSpec": ".posted_event_message_spec",
    "PostedEventMessageSpecs": ".posted_event_message_specs",
    "PostedLanguageModel": ".posted_language_model",
    "PostedNudgeSpec": ".posted_nudge_spec",
    "PostedTimeoutSpec": ".posted_timeout_spec",
    "PostedTimeoutSpecs": ".posted_timeout_specs",
    "PostedTimeoutSpecsInactivity": ".posted_timeout_specs_inactivity",
    "PostedTimeoutSpecsMaxDuration": ".posted_timeout_specs_max_duration",
    "PostedUserDefinedToolSpec": ".posted_user_defined_tool_spec",
    "PostedWebhookEventType": ".posted_webhook_event_type",
    "PostedWebhookSpec": ".posted_webhook_spec",
    "ProsodyInference": ".prosody_inference",
    "ResumeAssistantMessage": ".resume_assistant_message",
    "ReturnBuiltinTool": ".return_builtin_tool",
    "ReturnBuiltinToolToolType": ".return_builtin_tool_tool_type",
    "ReturnChat": ".return_chat",
    "ReturnChatAudioReconstruction": ".return_chat_audio_reconstruction",
    "ReturnChatAudioReconstructionStatus": ".return_chat_audio_reconstruction_status",
    "ReturnChatEvent": ".return_chat_event",
    "ReturnChatEventRole": ".return_chat_event_role",
    "ReturnChatEventType": ".return_chat_event_type",
    "ReturnChatGroup": ".return_chat_group",
    "ReturnChatGroupPagedAudioReconstructions": ".return_chat_group_paged_audio_reconstructions",
    "ReturnChatGroupPagedAudioReconstructionsPaginationDirection": ".return_chat_group_paged_audio_reconstructions_pagination_direction",
    "ReturnChatGroupPagedChats": ".return_chat_group_paged_chats",
    "ReturnChatGroupPagedChatsPaginationDirection": ".return_chat_group_paged_chats_pagination_direction",
    "ReturnChatGroupPagedEvents": ".return_chat_group_paged_events",
    "ReturnChatGroupPagedEventsPaginationDirection": ".return_chat_group_paged_events_pagination_direction",
    "ReturnChatPagedEvents": ".return_chat_paged_events",
    "ReturnChatPagedEventsPaginationDirection": ".return_chat_paged_events_pagination_direction",
    "ReturnChatPagedEventsStatus": ".return_chat_paged_events_status",
    "ReturnChatStatus": ".return_chat_status",
    "ReturnConfig": ".return_config",
    "ReturnConfigSpec": ".return_config_spec",
    "ReturnEllmModel": ".return_ellm_model",
    "ReturnEventMessageSpec": ".return_event_message_spec",
    "ReturnEventMessageSpecs": ".return_event_message_specs",
    "ReturnLanguageModel": ".return_language_model",
    "ReturnNudgeSpec": ".return_nudge_spec",
    "ReturnPagedChatGroups": ".return_paged_chat_groups",
    "ReturnPagedChatGroupsPaginationDirection": ".return_paged_chat_groups_pagination_direction",
    "ReturnPagedChats": ".return_paged_chats",
    "ReturnPagedChatsPaginationDirection": ".return_paged_chats_pagination_direction",
    "ReturnPagedConfigs": ".return_paged_configs",
    "ReturnPagedPrompts": ".return_paged_prompts",
    "ReturnPagedUserDefinedTools": ".return_paged_user_defined_tools",
    "ReturnPrompt": ".return_prompt",
    "ReturnPromptVersionType": ".return_prompt_version_type",
    "ReturnTimeoutSpec": ".return_timeout_spec",
    "ReturnTimeoutSpecs": ".return_timeout_specs",
    "ReturnUserDefinedTool": ".return_user_defined_tool",
    "ReturnUserDefinedToolToolType": ".return_user_defined_tool_tool_type",
    "ReturnUserDefinedToolVersionType": ".return_user_defined_tool_version_type",
    "ReturnVoice": ".return_voice",
    "ReturnWebhookEventType": ".return_webhook_event_type",
    "ReturnWebhookSpec": ".return_webhook_spec",
    "Role": ".role",
    "SessionSettings": ".session_settings",
    "SessionSettingsVariablesValue": ".session_settings_variables_value",
    "SubscribeEvent": ".subscribe_event",
    "Tool": ".tool",
    "ToolCallMessage": ".tool_call_message",
    "ToolErrorMessage": ".tool_error_message",
    "ToolResponseMessage": ".tool_response_message",
    "ToolType": ".tool_type",
    "UserInput": ".user_input",
    "UserInterruption": ".user_interruption",
    "UserMessage": ".user_message",
    "ValidationError": ".validation_error",
    "ValidationErrorLocItem": ".validation_error_loc_item",
    "VoiceId": ".voice_id",
    "VoiceName": ".voice_name",
    "VoiceProvider": ".voice_provider",
    "VoiceRef": ".voice_ref",
    "WebSocketError": ".web_socket_error",
    "WebhookEvent": ".webhook_event",
    "WebhookEventBase": ".webhook_event_base",
    "WebhookEventChatEnded": ".webhook_event_chat_ended",
    "WebhookEventChatStartType": ".webhook_event_chat_start_type",
    "WebhookEventChatStarted": ".webhook_event_chat_started",
    "WebhookEventChatStatus": ".webhook_event_chat_status",
    "WebhookEventToolCall": ".webhook_event_tool_call",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "AssistantEnd",
    "AssistantInput",
    "AssistantMessage",
    "AssistantProsody",
    "AudioConfiguration",
    "AudioInput",
    "AudioOutput",
    "BuiltInTool",
    "BuiltinToolConfig",
    "ChatMessage",
    "ChatMessageToolResult",
    "ChatMetadata",
    "ConnectSessionSettings",
    "ConnectSessionSettingsAudio",
    "ConnectSessionSettingsContext",
    "ConnectSessionSettingsVariablesValue",
    "Context",
    "ContextType",
    "ControlPlanePublishEvent",
    "EmotionScores",
    "Encoding",
    "ErrorLevel",
    "ErrorResponse",
    "HttpValidationError",
    "Inference",
    "JsonMessage",
    "LanguageModelType",
    "MillisecondInterval",
    "ModelProviderEnum",
    "PauseAssistantMessage",
    "PostedBuiltinTool",
    "PostedBuiltinToolName",
    "PostedConfigPromptSpec",
    "PostedEllmModel",
    "PostedEventMessageSpec",
    "PostedEventMessageSpecs",
    "PostedLanguageModel",
    "PostedNudgeSpec",
    "PostedTimeoutSpec",
    "PostedTimeoutSpecs",
    "PostedTimeoutSpecsInactivity",
    "PostedTimeoutSpecsMaxDuration",
    "PostedUserDefinedToolSpec",
    "PostedWebhookEventType",
    "PostedWebhookSpec",
    "ProsodyInference",
    "ResumeAssistantMessage",
    "ReturnBuiltinTool",
    "ReturnBuiltinToolToolType",
    "ReturnChat",
    "ReturnChatAudioReconstruction",
    "ReturnChatAudioReconstructionStatus",
    "ReturnChatEvent",
    "ReturnChatEventRole",
    "ReturnChatEventType",
    "ReturnChatGroup",
    "ReturnChatGroupPagedAudioReconstructions",
    "ReturnChatGroupPagedAudioReconstructionsPaginationDirection",
    "ReturnChatGroupPagedChats",
    "ReturnChatGroupPagedChatsPaginationDirection",
    "ReturnChatGroupPagedEvents",
    "ReturnChatGroupPagedEventsPaginationDirection",
    "ReturnChatPagedEvents",
    "ReturnChatPagedEventsPaginationDirection",
    "ReturnChatPagedEventsStatus",
    "ReturnChatStatus",
    "ReturnConfig",
    "ReturnConfigSpec",
    "ReturnEllmModel",
    "ReturnEventMessageSpec",
    "ReturnEventMessageSpecs",
    "ReturnLanguageModel",
    "ReturnNudgeSpec",
    "ReturnPagedChatGroups",
    "ReturnPagedChatGroupsPaginationDirection",
    "ReturnPagedChats",
    "ReturnPagedChatsPaginationDirection",
    "ReturnPagedConfigs",
    "ReturnPagedPrompts",
    "ReturnPagedUserDefinedTools",
    "ReturnPrompt",
    "ReturnPromptVersionType",
    "ReturnTimeoutSpec",
    "ReturnTimeoutSpecs",
    "ReturnUserDefinedTool",
    "ReturnUserDefinedToolToolType",
    "ReturnUserDefinedToolVersionType",
    "ReturnVoice",
    "ReturnWebhookEventType",
    "ReturnWebhookSpec",
    "Role",
    "SessionSettings",
    "SessionSettingsVariablesValue",
    "SubscribeEvent",
    "Tool",
    "ToolCallMessage",
    "ToolErrorMessage",
    "ToolResponseMessage",
    "ToolType",
    "UserInput",
    "UserInterruption",
    "UserMessage",
    "ValidationError",
    "ValidationErrorLocItem",
    "VoiceId",
    "VoiceName",
    "VoiceProvider",
    "VoiceRef",
    "WebSocketError",
    "WebhookEvent",
    "WebhookEventBase",
    "WebhookEventChatEnded",
    "WebhookEventChatStartType",
    "WebhookEventChatStarted",
    "WebhookEventChatStatus",
    "WebhookEventToolCall",
]



# This file was auto-generated by Fern from our API Definition.

import base64
import json
import typing
from json.decoder import JSONDecodeError
from pathlib import Path

import websockets
import websockets.sync.connection as websockets_sync_connection

from ....core.api_error import ApiError
from ....core.events import EventEmitterMixin, EventType
from ....core.pydantic_utilities import parse_obj_as
from .types.config import Config
from .types.stream_models_endpoint_payload import StreamModelsEndpointPayload
from .types.subscribe_event import SubscribeEvent

try:
    from websockets.legacy.client import WebSocketClientProtocol  # type: ignore
except ImportError:
    from websockets import WebSocketClientProtocol  # type: ignore

StreamSocketClientResponse = typing.Union[SubscribeEvent]


class AsyncStreamSocketClient(EventEmitterMixin):
    def __init__(self, *, websocket: WebSocketClientProtocol):
        super().__init__()
        self._websocket = websocket

    async def __aiter__(self):
        async for message in self._websocket:
            yield parse_obj_as(StreamSocketClientResponse, json.loads(message))  # type: ignore

    async def start_listening(self):
        """
        Start listening for messages on the websocket connection.

        Emits events in the following order:
        - EventType.OPEN when connection is established
        - EventType.MESSAGE for each message received
        - EventType.ERROR if an error occurs
        - EventType.CLOSE when connection is closed
        """
        await self._emit_async(EventType.OPEN, None)
        try:
            async for raw_message in self._websocket:
                json_data = json.loads(raw_message)
                parsed = parse_obj_as(StreamSocketClientResponse, json_data)  # type: ignore
                await self._emit_async(EventType.MESSAGE, parsed)
        except (websockets.WebSocketException, JSONDecodeError) as exc:
            await self._emit_async(EventType.ERROR, exc)
        finally:
            await self._emit_async(EventType.CLOSE, None)

    async def send_publish(self, message: StreamModelsEndpointPayload) -> None:
        """
        Send a message to the websocket connection.
        The message will be sent as a StreamModelsEndpointPayload.
        """
        await self._send_model(message)

    async def recv(self) -> StreamSocketClientResponse:
        """
        Receive a message from the websocket connection.
        """
        data = await self._websocket.recv()
        json_data = json.loads(data)
        return parse_obj_as(StreamSocketClientResponse, json_data)  # type: ignore

    async def _send(self, data: typing.Any) -> None:
        """
        Send a message to the websocket connection.
        """
        if isinstance(data, dict):
            data = json.dumps(data)
        await self._websocket.send(data)

    async def _send_model(self, data: typing.Any) -> None:
        """
        Send a Pydantic model to the websocket connection.
        """
        await self._send(data.dict())

    async def send_facemesh(
        self,
        landmarks: typing.List[typing.List[typing.List[float]]],
        config: typing.Optional[Config] = None,
        payload_id: typing.Optional[str] = None,
    ) -> StreamSocketClientResponse:
        landmarks_str = json.dumps(landmarks)
        payload = {
            "data": landmarks_str,
            "models": config.dict() if config else None,
            "raw_text": False,
            "payload_id": payload_id,
        }
        payload = {k: v for k, v in payload.items() if v is not None}
        await self._websocket.send(json.dumps(payload))
        return await self.recv()

    async def send_text(
        self,
        text: str,
        config: typing.Optional[Config] = None,
        payload_id: typing.Optional[str] = None,
    ) -> StreamSocketClientResponse:
        payload = {
            "data": text,
            "models": config.dict() if config else None,
            "raw_text": True,
            "payload_id": payload_id,
        }
        payload = {k: v for k, v in payload.items() if v is not None}
        await self._websocket.send(json.dumps(payload))
        return await self.recv()

    async def send_file(
        self,
        file_: typing.Union[str, Path],
        config: typing.Optional[Config] = None,
        payload_id: typing.Optional[str] = None,
    ) -> StreamSocketClientResponse:
        try:
            with open(file_, "rb") as f:
                bytes_data = base64.b64encode(f.read()).decode()
        except:
            if isinstance(file_, Path):
                raise ApiError(body=f"Failed to open file: {file_}")
            # If you cannot open the file, assume you were passed a b64 string, not a file path
            bytes_data = str(file_)

        payload = {
            "data": bytes_data,
            "models": config.dict() if config else None,
            "raw_text": False,
            "payload_id": payload_id,
        }
        payload = {k: v for k, v in payload.items() if v is not None}
        await self._websocket.send(json.dumps(payload))
        return await self.recv()

    async def get_job_details(self) -> StreamSocketClientResponse:
        payload = {"job_details": True}
        await self._websocket.send(json.dumps(payload))
        return await self.recv()

    async def reset(self) -> StreamSocketClientResponse:
        payload = {"reset_stream": True}
        await self._websocket.send(json.dumps(payload))
        return await self.recv()


class StreamSocketClient(EventEmitterMixin):
    def __init__(self, *, websocket: websockets_sync_connection.Connection):
        super().__init__()
        self._websocket = websocket

    def __iter__(self):
        for message in self._websocket:
            yield parse_obj_as(StreamSocketClientResponse, json.loads(message))  # type: ignore

    def start_listening(self):
        """
        Start listening for messages on the websocket connection.

        Emits events in the following order:
        - EventType.OPEN when connection is established
        - EventType.MESSAGE for each message received
        - EventType.ERROR if an error occurs
        - EventType.CLOSE when connection is closed
        """
        self._emit(EventType.OPEN, None)
        try:
            for raw_message in self._websocket:
                json_data = json.loads(raw_message)
                parsed = parse_obj_as(StreamSocketClientResponse, json_data)  # type: ignore
                self._emit(EventType.MESSAGE, parsed)
        except (websockets.WebSocketException, JSONDecodeError) as exc:
            self._emit(EventType.ERROR, exc)
        finally:
            self._emit(EventType.CLOSE, None)

    def send_publish(self, message: StreamModelsEndpointPayload) -> None:
        """
        Send a message to the websocket connection.
        The message will be sent as a StreamModelsEndpointPayload.
        """
        self._send_model(message)

    def recv(self) -> StreamSocketClientResponse:
        """
        Receive a message from the websocket connection.
        """
        data = self._websocket.recv()
        json_data = json.loads(data)
        return parse_obj_as(StreamSocketClientResponse, json_data)  # type: ignore

    def _send(self, data: typing.Any) -> None:
        """
        Send a message to the websocket connection.
        """
        if isinstance(data, dict):
            data = json.dumps(data)
        self._websocket.send(data)

    def _send_model(self, data: typing.Any) -> None:
        """
        Send a Pydantic model to the websocket connection.
        """
        self._send(data.dict())

    def send_facemesh(
        self,
        landmarks: typing.List[typing.List[typing.List[float]]],
        config: typing.Optional[Config] = None,
        payload_id: typing.Optional[str] = None,
    ) -> StreamSocketClientResponse:
        landmarks_str = json.dumps(landmarks)
        payload = {
            "data": landmarks_str,
            "models": config.dict() if config else None,
            "raw_text": False,
            "payload_id": payload_id,
        }
        payload = {k: v for k, v in payload.items() if v is not None}
        self._websocket.send(json.dumps(payload))
        return self.recv()

    def send_text(
        self,
        text: str,
        config: typing.Optional[Config] = None,
        payload_id: typing.Optional[str] = None,
    ) -> StreamSocketClientResponse:
        payload = {
            "data": text,
            "models": config.dict() if config else None,
            "raw_text": True,
            "payload_id": payload_id,
        }
        payload = {k: v for k, v in payload.items() if v is not None}
        self._websocket.send(json.dumps(payload))
        return self.recv()

    def send_file(
        self,
        file_: typing.Union[str, Path],
        config: typing.Optional[Config] = None,
        payload_id: typing.Optional[str] = None,
    ) -> StreamSocketClientResponse:
        try:
            with open(file_, "rb") as f:
                bytes_data = base64.b64encode(f.read()).decode()
        except:
            if isinstance(file_, Path):
                raise ApiError(body=f"Failed to open file: {file_}")
            # If you cannot open the file, assume you were passed a b64 string, not a file path
            bytes_data = str(file_)

        payload = {
            "data": bytes_data,
            "models": config.dict() if config else None,
            "raw_text": False,
            "payload_id": payload_id,
        }
        payload = {k: v for k, v in payload.items() if v is not None}
        self._websocket.send(json.dumps(payload))
        return self.recv()

    def get_job_details(self) -> StreamSocketClientResponse:
        payload = {"job_details": True}
        self._websocket.send(json.dumps(payload))
        return self.recv()

    def reset(self) -> StreamSocketClientResponse:
        payload = {"reset_stream": True}
        self._websocket.send(json.dumps(payload))
        return self.recv()



# This file was auto-generated by Fern from our API Definition.

import typing

from .snippet_audio_chunk import SnippetAudioChunk
from .timestamp_message import TimestampMessage

TtsOutput = typing.Union[SnippetAudioChunk, TimestampMessage]



# This file was auto-generated by Fern from our API Definition.

try:
    from websockets.exceptions import InvalidStatusCode as InvalidWebSocketStatus  # type: ignore[attr-defined]

    def get_status_code(exc: InvalidWebSocketStatus) -> int:
        return exc.status_code  # type: ignore[attr-defined, union-attr]

except ImportError:
    from websockets.exceptions import InvalidStatus as InvalidWebSocketStatus  # type: ignore[assignment, no-redef]

    def get_status_code(exc: InvalidWebSocketStatus) -> int:  # type: ignore[no-redef]
        return exc.response.status_code  # type: ignore[attr-defined, union-attr]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .millisecond_interval import MillisecondInterval
from .timestamp_type import TimestampType


class Timestamp(UniversalBaseModel):
    text: str = pydantic.Field()
    """
    The word or phoneme text that the timestamp corresponds to.
    """

    time: MillisecondInterval = pydantic.Field()
    """
    The start and end timestamps for the word or phoneme in milliseconds.
    """

    type: TimestampType

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .encoding import Encoding


class ConnectSessionSettingsAudio(UniversalBaseModel):
    """
    Configuration details for the audio input used during the session. Ensures the audio is being correctly set up for processing.

    This optional field is only required when the audio input is encoded in PCM Linear 16 (16-bit, little-endian, signed PCM WAV data). For detailed instructions on how to configure session settings for PCM Linear 16 audio, please refer to the [Session Settings section](/docs/empathic-voice-interface-evi/configuration#session-settings) on the EVI Configuration page.
    """

    channels: typing.Optional[int] = pydantic.Field(default=None)
    """
    Sets number of audio channels for audio input.
    """

    encoding: typing.Optional[Encoding] = pydantic.Field(default=None)
    """
    Sets encoding format of the audio input, such as `linear16`.
    """

    sample_rate: typing.Optional[int] = pydantic.Field(default=None)
    """
    Sets the sample rate for audio input. (Number of samples per second in the audio input, measured in Hertz.)
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .language_model_type import LanguageModelType
from .model_provider_enum import ModelProviderEnum


class PostedLanguageModel(UniversalBaseModel):
    """
    A LanguageModel to be posted to the server
    """

    model_provider: typing.Optional[ModelProviderEnum] = None
    model_resource: typing.Optional[LanguageModelType] = None
    temperature: typing.Optional[float] = pydantic.Field(default=None)
    """
    The model temperature, with values between 0 to 1 (inclusive).
    
    Controls the randomness of the LLM's output, with values closer to 0 yielding focused, deterministic responses and values closer to 1 producing more creative, diverse responses.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .audio_format_type import AudioFormatType
from .snippet import Snippet


class SnippetAudioChunk(UniversalBaseModel):
    """
    Metadata for a chunk of generated audio.
    """

    audio: str = pydantic.Field()
    """
    The generated audio output chunk in the requested format.
    """

    audio_format: AudioFormatType = pydantic.Field()
    """
    The generated audio output format.
    """

    chunk_index: int = pydantic.Field()
    """
    The index of the audio chunk in the snippet.
    """

    generation_id: str = pydantic.Field()
    """
    The generation ID of the parent snippet that this chunk corresponds to.
    """

    is_last_chunk: bool = pydantic.Field()
    """
    Whether or not this is the last chunk streamed back from the decoder for one input snippet.
    """

    request_id: str = pydantic.Field()
    """
    ID of the initiating request.
    """

    snippet: typing.Optional[Snippet] = None
    snippet_id: str = pydantic.Field()
    """
    The ID of the parent snippet that this chunk corresponds to.
    """

    text: str = pydantic.Field()
    """
    The text of the parent snippet that this chunk corresponds to.
    """

    transcribed_text: typing.Optional[str] = pydantic.Field(default=None)
    """
    The transcribed text of the generated audio of the parent snippet that this chunk corresponds to. It is only present if `instant_mode` is set to `false`.
    """

    type: typing.Literal["audio"] = "audio"
    utterance_index: typing.Optional[int] = pydantic.Field(default=None)
    """
    The index of the utterance in the request that the parent snippet of this chunk corresponds to.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from typing_extensions import deprecated

import pydantic
from ....core.pydantic_utilities import IS_PYDANTIC_V2
from .job_inference import JobInference


class InferenceJob(JobInference):
    type: str = pydantic.Field()
    """
    Denotes the job type.
    
    Jobs created with the Expression Measurement API will have this field set to `INFERENCE`.
    """

    @property
    @deprecated("Use .state.status instead")
    def status(self) -> str:
        return self.state.status

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class AudioInput(UniversalBaseModel):
    """
    **Base64 encoded audio input to insert into the conversation.** The content is treated as the user's speech to EVI and must be streamed continuously. Pre-recorded audio files are not supported.

    For optimal transcription quality, the audio data should be transmitted in small chunks. Hume recommends streaming audio with a buffer window of `20` milliseconds (ms), or `100` milliseconds (ms) for web applications. See our [Audio Guide](/docs/speech-to-speech-evi/guides/audio) for more details on preparing and processing audio.
    """

    custom_session_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Used to manage conversational state, correlate frontend and backend data, and persist conversations across EVI sessions.
    """

    data: str = pydantic.Field()
    """
    Base64 encoded audio input to insert into the conversation.
    
    The content of an Audio Input message is treated as the user's speech to EVI and must be streamed continuously. Pre-recorded audio files are not supported.
    
    For optimal transcription quality, the audio data should be transmitted in small chunks.
    
    Hume recommends streaming audio with a buffer window of 20 milliseconds (ms), or 100 milliseconds (ms) for web applications.
    """

    type: typing.Literal["audio_input"] = pydantic.Field(default="audio_input")
    """
    The type of message sent through the socket; must be `audio_input` for our server to correctly identify and process it as an Audio Input message.
    
    This message is used for sending audio input data to EVI for processing and expression measurement. Audio data should be sent as a continuous stream, encoded in Base64.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from .....core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .stream_model_predictions_burst import StreamModelPredictionsBurst
from .stream_model_predictions_face import StreamModelPredictionsFace
from .stream_model_predictions_facemesh import StreamModelPredictionsFacemesh
from .stream_model_predictions_job_details import StreamModelPredictionsJobDetails
from .stream_model_predictions_language import StreamModelPredictionsLanguage
from .stream_model_predictions_prosody import StreamModelPredictionsProsody


class StreamModelPredictions(UniversalBaseModel):
    """
    Model predictions
    """

    payload_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    If a payload ID was passed in the request, the same payload ID will be sent back in the response body.
    """

    job_details: typing.Optional[StreamModelPredictionsJobDetails] = pydantic.Field(default=None)
    """
    If the job_details flag was set in the request, details about the current streaming job will be returned in the response body.
    """

    burst: typing.Optional[StreamModelPredictionsBurst] = pydantic.Field(default=None)
    """
    Response for the vocal burst emotion model.
    """

    face: typing.Optional[StreamModelPredictionsFace] = pydantic.Field(default=None)
    """
    Response for the facial expression emotion model.
    """

    facemesh: typing.Optional[StreamModelPredictionsFacemesh] = pydantic.Field(default=None)
    """
    Response for the facemesh emotion model.
    """

    language: typing.Optional[StreamModelPredictionsLanguage] = pydantic.Field(default=None)
    """
    Response for the language emotion model.
    """

    prosody: typing.Optional[StreamModelPredictionsProsody] = pydantic.Field(default=None)
    """
    Response for the speech prosody emotion model.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ....core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .registry_file_detail import RegistryFileDetail


class EmbeddingGenerationBaseRequest(UniversalBaseModel):
    registry_file_details: typing.Optional[typing.List[RegistryFileDetail]] = pydantic.Field(default=None)
    """
    File ID and File URL pairs for an asset registry file
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class ResumeAssistantMessage(UniversalBaseModel):
    """
    **Resume responses from EVI.** Chat history sent while paused will now be sent.

    Upon resuming, if any audio input was sent during the pause, EVI will retain context from all messages sent but only respond to the last user message. See our [Pause Response Guide](/docs/speech-to-speech-evi/features/pause-responses) for further details.
    """

    custom_session_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Used to manage conversational state, correlate frontend and backend data, and persist conversations across EVI sessions.
    """

    type: typing.Literal["resume_assistant_message"] = pydantic.Field(default="resume_assistant_message")
    """
    The type of message sent through the socket; must be `resume_assistant_message` for our server to correctly identify and process it as a Resume Assistant message.
    
    Upon resuming, if any audio input was sent during the pause, EVI will retain context from all messages sent but only respond to the last user message. (e.g., If you ask EVI two questions while paused and then send a `resume_assistant_message`, EVI will respond to the second question and have added the first question to its conversation context.)
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .config import Config
    from .job_details import JobDetails
    from .stream_error_message import StreamErrorMessage
    from .stream_face import StreamFace
    from .stream_language import StreamLanguage
    from .stream_model_predictions import StreamModelPredictions
    from .stream_model_predictions_burst import StreamModelPredictionsBurst
    from .stream_model_predictions_burst_predictions_item import StreamModelPredictionsBurstPredictionsItem
    from .stream_model_predictions_face import StreamModelPredictionsFace
    from .stream_model_predictions_face_predictions_item import StreamModelPredictionsFacePredictionsItem
    from .stream_model_predictions_facemesh import StreamModelPredictionsFacemesh
    from .stream_model_predictions_facemesh_predictions_item import StreamModelPredictionsFacemeshPredictionsItem
    from .stream_model_predictions_job_details import StreamModelPredictionsJobDetails
    from .stream_model_predictions_language import StreamModelPredictionsLanguage
    from .stream_model_predictions_language_predictions_item import StreamModelPredictionsLanguagePredictionsItem
    from .stream_model_predictions_prosody import StreamModelPredictionsProsody
    from .stream_model_predictions_prosody_predictions_item import StreamModelPredictionsProsodyPredictionsItem
    from .stream_models_endpoint_payload import StreamModelsEndpointPayload
    from .stream_warning_message import StreamWarningMessage
    from .stream_warning_message_job_details import StreamWarningMessageJobDetails
    from .subscribe_event import SubscribeEvent
_dynamic_imports: typing.Dict[str, str] = {
    "Config": ".config",
    "JobDetails": ".job_details",
    "StreamErrorMessage": ".stream_error_message",
    "StreamFace": ".stream_face",
    "StreamLanguage": ".stream_language",
    "StreamModelPredictions": ".stream_model_predictions",
    "StreamModelPredictionsBurst": ".stream_model_predictions_burst",
    "StreamModelPredictionsBurstPredictionsItem": ".stream_model_predictions_burst_predictions_item",
    "StreamModelPredictionsFace": ".stream_model_predictions_face",
    "StreamModelPredictionsFacePredictionsItem": ".stream_model_predictions_face_predictions_item",
    "StreamModelPredictionsFacemesh": ".stream_model_predictions_facemesh",
    "StreamModelPredictionsFacemeshPredictionsItem": ".stream_model_predictions_facemesh_predictions_item",
    "StreamModelPredictionsJobDetails": ".stream_model_predictions_job_details",
    "StreamModelPredictionsLanguage": ".stream_model_predictions_language",
    "StreamModelPredictionsLanguagePredictionsItem": ".stream_model_predictions_language_predictions_item",
    "StreamModelPredictionsProsody": ".stream_model_predictions_prosody",
    "StreamModelPredictionsProsodyPredictionsItem": ".stream_model_predictions_prosody_predictions_item",
    "StreamModelsEndpointPayload": ".stream_models_endpoint_payload",
    "StreamWarningMessage": ".stream_warning_message",
    "StreamWarningMessageJobDetails": ".stream_warning_message_job_details",
    "SubscribeEvent": ".subscribe_event",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "Config",
    "JobDetails",
    "StreamErrorMessage",
    "StreamFace",
    "StreamLanguage",
    "StreamModelPredictions",
    "StreamModelPredictionsBurst",
    "StreamModelPredictionsBurstPredictionsItem",
    "StreamModelPredictionsFace",
    "StreamModelPredictionsFacePredictionsItem",
    "StreamModelPredictionsFacemesh",
    "StreamModelPredictionsFacemeshPredictionsItem",
    "StreamModelPredictionsJobDetails",
    "StreamModelPredictionsLanguage",
    "StreamModelPredictionsLanguagePredictionsItem",
    "StreamModelPredictionsProsody",
    "StreamModelPredictionsProsodyPredictionsItem",
    "StreamModelsEndpointPayload",
    "StreamWarningMessage",
    "StreamWarningMessageJobDetails",
    "SubscribeEvent",
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .audio_encoding import AudioEncoding
    from .audio_format_type import AudioFormatType
    from .error_response import ErrorResponse
    from .format import Format
    from .format_mp_3 import FormatMp3
    from .format_pcm import FormatPcm
    from .format_wav import FormatWav
    from .http_validation_error import HttpValidationError
    from .millisecond_interval import MillisecondInterval
    from .octave_version import OctaveVersion
    from .posted_context import PostedContext
    from .posted_context_with_generation_id import PostedContextWithGenerationId
    from .posted_context_with_utterances import PostedContextWithUtterances
    from .posted_tts import PostedTts
    from .posted_utterance import PostedUtterance
    from .posted_utterance_voice import PostedUtteranceVoice
    from .posted_utterance_voice_with_id import PostedUtteranceVoiceWithId
    from .posted_utterance_voice_with_name import PostedUtteranceVoiceWithName
    from .publish_tts import PublishTts
    from .return_generation import ReturnGeneration
    from .return_paged_voices import ReturnPagedVoices
    from .return_tts import ReturnTts
    from .return_voice import ReturnVoice
    from .snippet import Snippet
    from .snippet_audio_chunk import SnippetAudioChunk
    from .timestamp import Timestamp
    from .timestamp_message import TimestampMessage
    from .timestamp_type import TimestampType
    from .tts_output import TtsOutput
    from .validation_error import ValidationError
    from .validation_error_loc_item import ValidationErrorLocItem
    from .voice_provider import VoiceProvider
_dynamic_imports: typing.Dict[str, str] = {
    "AudioEncoding": ".audio_encoding",
    "AudioFormatType": ".audio_format_type",
    "ErrorResponse": ".error_response",
    "Format": ".format",
    "FormatMp3": ".format_mp_3",
    "FormatPcm": ".format_pcm",
    "FormatWav": ".format_wav",
    "HttpValidationError": ".http_validation_error",
    "MillisecondInterval": ".millisecond_interval",
    "OctaveVersion": ".octave_version",
    "PostedContext": ".posted_context",
    "PostedContextWithGenerationId": ".posted_context_with_generation_id",
    "PostedContextWithUtterances": ".posted_context_with_utterances",
    "PostedTts": ".posted_tts",
    "PostedUtterance": ".posted_utterance",
    "PostedUtteranceVoice": ".posted_utterance_voice",
    "PostedUtteranceVoiceWithId": ".posted_utterance_voice_with_id",
    "PostedUtteranceVoiceWithName": ".posted_utterance_voice_with_name",
    "PublishTts": ".publish_tts",
    "ReturnGeneration": ".return_generation",
    "ReturnPagedVoices": ".return_paged_voices",
    "ReturnTts": ".return_tts",
    "ReturnVoice": ".return_voice",
    "Snippet": ".snippet",
    "SnippetAudioChunk": ".snippet_audio_chunk",
    "Timestamp": ".timestamp",
    "TimestampMessage": ".timestamp_message",
    "TimestampType": ".timestamp_type",
    "TtsOutput": ".tts_output",
    "ValidationError": ".validation_error",
    "ValidationErrorLocItem": ".validation_error_loc_item",
    "VoiceProvider": ".voice_provider",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "AudioEncoding",
    "AudioFormatType",
    "ErrorResponse",
    "Format",
    "FormatMp3",
    "FormatPcm",
    "FormatWav",
    "HttpValidationError",
    "MillisecondInterval",
    "OctaveVersion",
    "PostedContext",
    "PostedContextWithGenerationId",
    "PostedContextWithUtterances",
    "PostedTts",
    "PostedUtterance",
    "PostedUtteranceVoice",
    "PostedUtteranceVoiceWithId",
    "PostedUtteranceVoiceWithName",
    "PublishTts",
    "ReturnGeneration",
    "ReturnPagedVoices",
    "ReturnTts",
    "ReturnVoice",
    "Snippet",
    "SnippetAudioChunk",
    "Timestamp",
    "TimestampMessage",
    "TimestampType",
    "TtsOutput",
    "ValidationError",
    "ValidationErrorLocItem",
    "VoiceProvider",
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
import typing_extensions
from ....core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ....core.serialization import FieldMetadata


class Source_Url(UniversalBaseModel):
    type: typing.Literal["url"] = "url"
    url: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class Source_File(UniversalBaseModel):
    type: typing.Literal["file"] = "file"
    filename: typing.Optional[str] = None
    content_type: typing.Optional[str] = None
    md_5_sum: typing_extensions.Annotated[str, FieldMetadata(alias="md5sum"), pydantic.Field(alias="md5sum")]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class Source_Text(UniversalBaseModel):
    type: typing.Literal["text"] = "text"

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


Source = typing_extensions.Annotated[
    typing.Union[Source_Url, Source_File, Source_Text], pydantic.Field(discriminator="type")
]
